package com.hzya.frame.plugin.cbs8.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.cbs8.dto.req.ElecRequestDTO;
import com.hzya.frame.cbs8.dto.req.PayResultRequestDTO;
import com.hzya.frame.cbs8.dto.req.TransactionDetailReqDTO;
import com.hzya.frame.cbs8.dto.res.*;
import com.hzya.frame.cbs8.service.ICbs8Service;
import com.hzya.frame.cbs8.util.CBSUtil;
import com.hzya.frame.cbs8.util.PayState;
import com.hzya.frame.plugin.cbs8.service.ICbsPluginService;
import com.hzya.frame.seeyon.cbs8.entity.CbsLogEntity;
import com.hzya.frame.seeyon.cbs8.entity.PaymentEntity;
import com.hzya.frame.seeyon.cbs8.service.ICbsLogService;
import com.hzya.frame.seeyon.cbs8.service.IPaymentService;
import com.hzya.frame.seeyon.entity.CtpAttachmentEntity;
import com.hzya.frame.seeyon.entity.CtpFileEntity;
import com.hzya.frame.seeyon.service.ICtpAttachmentService;
import com.hzya.frame.seeyon.util.RestUtil;
import com.hzya.frame.uuid.UUIDLong;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * @Description
 * @Author xiangerlin
 * @Date 2024/6/17 08:46
 **/
public class CbsPluginServiceImpl implements ICbsPluginService {

    Logger logger = LoggerFactory.getLogger(CbsPluginServiceImpl.class);
    @Autowired
    private ICbs8Service cbs8Service;
    @Autowired
    private IPaymentService paymentService;
    @Autowired
    private ICbsLogService cbsLogService;
    @Autowired
    private ICtpAttachmentService ctpAttachmentService;

    @Value("${cbs8.elec_path}")
    private String elec_path;
    @Value("${OA.data_source_code}")
    private String oa_data_source_code;
    /**
     * 支付申请
     *
     * @param requestJson
     */
    @Override
    public void applyPay(JSONObject requestJson) throws Exception{
        //查询待支付的列表
        PaymentEntity paymentEntity = new PaymentEntity();
        paymentEntity.setDataSourceCode("");
        //List<PaymentEntity> paymentList = paymentService.queryUnpaid(paymentEntity);
        List<PaymentEntity> paymentList = new ArrayList<>();
        paymentEntity.setReferenceNum("CL202406140001");
        paymentEntity.setPayAccount("655905707410000");
        paymentEntity.setPayBankName("");
        paymentEntity.setAmount("99");
        paymentEntity.setRevAccount("123456778");
        paymentEntity.setRevBankName("中国工商银行总行清算中心");
        paymentEntity.setRevBankType("ICB");
        paymentEntity.setRevAccountName("测试账户");
        paymentEntity.setCnapsCode("102100099996");
        paymentEntity.setPurpose("测试用途");
        paymentEntity.setBusType("202");
        paymentEntity.setCurrency("10");
        paymentEntity.setPurpose("测试用途");
        paymentList.add(paymentEntity);
        if (CollectionUtils.isNotEmpty(paymentList)){
            for (PaymentEntity pay : paymentList) {
                //调用支付申请接口
                PayResponseDTO payResponseDTO = cbs8Service.payApply(pay);
                //4、更新OA表单
                paymentService.updatePayState(pay);
                //5、记录操作日志
                cbsLogService.saveLog(new CbsLogEntity());
            }
        }
    }

    /**
     * 查询支付申请的交易结果
     *
     * @param requestJson
     * @throws Exception
     */
    @Override
    public void queryResult(JSONObject requestJson) throws Exception {
        CbsLogEntity cbsLogEntity = new CbsLogEntity();
        // 1、查询支付中的日志
        List<CbsLogEntity> inPayList = cbsLogService.queryInPayment(cbsLogEntity);
        if (CollectionUtils.isNotEmpty(inPayList)){
            for (CbsLogEntity entity : inPayList) {
                try {
                    List<PayResultResDTO> payResultResList = cbs8Service.queryPayResult(new PayResultRequestDTO(entity.getBill_code()));
                    if (CollectionUtils.isNotEmpty(payResultResList)){
                        PayResultResDTO payResultResDTO = payResultResList.get(0);
                        //支付申请状态
                        String status = payResultResDTO.getStatus();
                        //支付状态
                        String pay_status = payResultResDTO.getPayStatus();
                        //不等于支付中的时候 更新支付状态
                        if (!PayState.p.getType().equals(pay_status)){
                            //如果支付状态为空，保存支付申请状态，如果支付状态不为空，则保存支付状态
                            PaymentEntity paymentEntity = new PaymentEntity();
                            paymentEntity.setOaId(entity.getOa_id());
                            List<PaymentEntity> paymentList = paymentService.query(paymentEntity);
                            if (CollectionUtils.isNotEmpty(paymentList)){
                                paymentEntity = paymentList.get(0);
                                if (StrUtil.isEmpty(pay_status)) {
                                    //支付申请状态 支付状态和支付申请状态用一个
                                    paymentEntity.setPayResult(PayState.payStateGetValue(status));
                                } else {
                                    //支付状态 支付状态和支付申请状态用一个
                                    paymentEntity.setPayResult(PayState.payStateGetValue(pay_status));
                                }
                                if (StrUtil.isNotEmpty(pay_status) && pay_status.equals(PayState.g.getType())) {
                                    //支付时间
                                    paymentEntity.setPayDate(CBSUtil.convertTimestampToString(payResultResDTO.getPayDate()));
                                }
                                //更新视图单据状态
                                paymentService.updatePayState(paymentEntity);
                                //更新日志表状态
                                entity.setPay_state(paymentEntity.getPayResult());
                                entity.setApply_state(PayState.payStateGetValue(status));
                                cbsLogService.update(entity);
                            }
                        }
                    }
                }catch (Exception e){
                 e.printStackTrace();
                 logger.error("查询交易结果出错",e);
                }
            }
        }
    }

    /**
     * 电子回单查询 并上传OA
     *
     * @param requestJson
     * @throws Exception
     */
    @Override
    public void elecBillUpload(JSONObject requestJson) throws Exception {
        //查询支付成功 没有电子回单的数据
        PaymentEntity paymentEntity = new PaymentEntity();
        List<PaymentEntity> paymentList = paymentService.queryElecIsNull(paymentEntity);
        if (CollectionUtils.isNotEmpty(paymentList)) {
            for (PaymentEntity pay : paymentList) {
                try {
                    String payDate = DateUtil.format(DateUtil.parse(pay.getPayDate()), "yyyy-MM-dd");
                    //查询cbs电子回单
                    List<ElecResponseDTO> elecResList = cbs8Service.queryElecBill(new ElecRequestDTO(payDate,DateUtil.today(),pay.getReferenceNum()));
                    if (CollectionUtils.isNotEmpty(elecResList)){
                        ElecResponseDTO elecResponseDTO = elecResList.get(0);
                        String bucketFileUrl = elecResponseDTO.getBucketFileUrl();
                        String bucketFileName = elecResponseDTO.getBucketFileName();
                        //上传电子回单到OA
                        HttpUtil.downloadFile(bucketFileUrl, FileUtil.file(elec_path));//附件下载
                        String pdfUrl = elec_path + bucketFileName;
                        File file = new File(pdfUrl);
                        if (file.exists()) {
                            CtpFileEntity cpFileEntity =  new CtpFileEntity();
                            cpFileEntity.setFile(file);
                            cpFileEntity.setDataSourceCode(oa_data_source_code);
                            JSONObject jsonObjectUpload = RestUtil.fileUpload(file,"8000240005");
                            String file_url = jsonObjectUpload.getString("fileUrl");
                            if (null != jsonObjectUpload && StrUtil.isNotEmpty(file_url)) {
                                String sub_reference = String.valueOf(UUIDLong.longUUID());
                                pay.setReceipt(sub_reference);
                                //更新表单的电子回单值
                                paymentService.updateElec(pay);
                                //保存附件关系
                                CtpAttachmentEntity ctpAttachmentEntity = new CtpAttachmentEntity();
                                ctpAttachmentEntity.setFile_url(file_url);
                                ctpAttachmentService.saveAttachment(file_url, pay.getSummaryId(), sub_reference);
                            }
                            //删除本地临时文件
                            file.delete();
                        }
                    }
                }catch (Exception e){
                    logger.error("电子回单查询出错",e);
                }
            }
        }
    }

    /**
     * 查询交易明细
     * transactionDetailReqDTO.currentPage 、 transactionDetailReqDTO.pageSize 必填
     * @param transactionDetailReqDTO
     */
    @Override
    public List<TransactionDetailDTO> queryTransactionDetail(TransactionDetailReqDTO transactionDetailReqDTO) {
        boolean hasNextPage = true;//是否有下一页
        int currentPage = transactionDetailReqDTO.getCurrentPage();
        int pageSize = transactionDetailReqDTO.getPageSize();
        if (currentPage == 0){
            currentPage = CBSUtil.DEFAULT_CURRENT_PAGE;//页码
        }
        if (pageSize == 0){
            pageSize = CBSUtil.DEFAULT_PAGE_SIZE;//每页条数
            transactionDetailReqDTO.setPageSize(pageSize);
        }
        List<TransactionDetailDTO> resultList = new ArrayList<>();
        do{
            transactionDetailReqDTO.setCurrentPage(currentPage);//页码
            currentPage++;//页码自增
            hasNextPage = false;//保护功能，防止出现死循环
            CbsResDataDTO dataDTO = cbs8Service.queryTransactionDetail(transactionDetailReqDTO);
            if (null != dataDTO){
                hasNextPage = dataDTO.getHasNextPage();
                List<TransactionDetailDTO> transactionDetailDTOList = CBSUtil.convertJsonArrayToList(dataDTO.getList(), TransactionDetailDTO.class);
                resultList.addAll(transactionDetailDTOList);
            }
        }while (hasNextPage);

        return resultList;
    }
}
