package com.hzya.frame.cbs8.service.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.cbs8.dto.req.ElecRequestDTO;
import com.hzya.frame.cbs8.dto.req.PayRequestDTO;
import com.hzya.frame.cbs8.dto.req.PayResultRequestDTO;
import com.hzya.frame.cbs8.dto.req.TransactionDetailReqDTO;
import com.hzya.frame.cbs8.service.ICbs8ExtService;
import com.hzya.frame.cbs8.util.CBSUtil;
import com.hzya.frame.cbs8.util.CbsAccessToken;
import com.hzya.frame.cbs8.util.SM2Util;
import com.hzya.frame.seeyon.cbs8.entity.PaymentEntity;
import com.hzya.frame.stringutil.StringUtil;
import com.hzya.frame.sysnew.application.entity.SysExtensionApiEntity;
import com.hzya.frame.web.exception.BaseSystemException;
import org.apache.http.protocol.HTTP;
import org.springframework.stereotype.Service;

import java.util.Base64;
import java.util.HashMap;
import java.util.Map;

/**
 * @Description
 * @Author xiangerlin
 * @Date 2024/6/7 14:20
 **/

@Service(value = "cbs8Ext")
public class Cbs8ExtServiceImpl implements ICbs8ExtService {

    /**
     * 支付经办
     *
     * @param entity
     * @return
     */
    @Override
    public SysExtensionApiEntity payApply(SysExtensionApiEntity entity) {
        String bodys = entity.getBodys();
        if (StrUtil.isNotEmpty(bodys)){
            PaymentEntity paymentEntity = JSONObject.parseObject(bodys,PaymentEntity.class);
            if (null != paymentEntity){
                PayRequestDTO payRequestDTO = new PayRequestDTO();
                String ferenceNum = Convert.toStr(paymentEntity.getReferenceNumNew(),paymentEntity.getReferenceNum());
                payRequestDTO.setReferenceNum(ferenceNum);
                payRequestDTO.setBusType(paymentEntity.getBusType());
                payRequestDTO.setAmount(paymentEntity.getAmount());
                payRequestDTO.setCurrency(paymentEntity.getCurrency());
                payRequestDTO.setPayAccount(StringUtil.replaceBlank(paymentEntity.getPayAccount()));
                payRequestDTO.setRevAccount(StringUtil.replaceBlank(paymentEntity.getRevAccount()));
                payRequestDTO.setRevAccountName(StringUtil.replaceBlank(paymentEntity.getRevAccountName()));
                payRequestDTO.setRevBankType(StringUtil.replaceBlank((paymentEntity.getRevBankType())));
                payRequestDTO.setRevBankName(StringUtil.replaceBlank(paymentEntity.getRevBankName()));
                payRequestDTO.setCnapsCode(StringUtil.replaceBlank((paymentEntity.getCnapsCode())));
                payRequestDTO.setPurpose(paymentEntity.getPurpose());
                payRequestDTO.setErpExtend1(paymentEntity.getPayType());
                //集中支付模式
                if (CBSUtil.CENTRALIZED_PAYMENT_TYPE.equals(payRequestDTO.getBusType())){
                    payRequestDTO.setBusiStep("1");
                    payRequestDTO.setApplyUnitCode(paymentEntity.getPayCompanyCode());
                    payRequestDTO.setPayAccount(null);
                }
                String requestData = JSONObject.toJSONString(payRequestDTO);
                //签名
                long timestamp = System.currentTimeMillis();
                String sign = CBSUtil.sign(requestData,timestamp);
                //加密
                byte[] encryptedData = CBSUtil.encrypt(requestData);
                Map<String,String> header = new HashMap();
                header.put(CBSUtil.SIGN_HEADER_NAME,sign);
                header.put(CBSUtil.TIMESTAMP_HEADER,Long.toString(timestamp));
                header.put(HTTP.CONTENT_TYPE,CBSUtil.TARGET_CONTENT_TYPE);
                header.put(CBSUtil.AUTHORIZATION,CBSUtil.BEARER+ CbsAccessToken.getToken());
                entity.setBodys(Base64.getEncoder().encodeToString(encryptedData));
                entity.setHeaders(header);

                String body = HttpRequest.post("https://cbs8-openapi-reprd.csuat.cmburl.cn/openapi/payment/openapi/v1/payment-apply-common").addHeaders(header).body(encryptedData).execute().body();
                System.out.println(body);
            }
        }
        return entity;
    }

    /**
     * 交易结果查询
     *
     * @param entity
     * @return
     */
    @Override
    public SysExtensionApiEntity payResult(SysExtensionApiEntity entity) {
        String bodys = entity.getBodys();
        if (StrUtil.isNotEmpty(bodys)){
            PayResultRequestDTO payResultRequest = JSONObject.parseObject(bodys,PayResultRequestDTO.class);
            if (null != payResultRequest && StrUtil.isNotEmpty(payResultRequest.getReferenceNum())){
                String requestData = JSONObject.toJSONString(payResultRequest);
                //签名
                long timestamp = System.currentTimeMillis();
                String sign = CBSUtil.sign(requestData,timestamp);
                //加密
                byte[] encryptedData = CBSUtil.encrypt(requestData);
                Map<String,String> header = getHeaders(sign,timestamp);
                entity.setBodys(Base64.getEncoder().encodeToString(encryptedData));
                entity.setHeaders(header);
                byte[] bodyBytes = HttpRequest.post("https://cbs8-openapi-reprd.csuat.cmburl.cn/openapi/payment/openapi/v2/detail").addHeaders(header).body(encryptedData).execute().bodyBytes();
                //解密报文
                String decrypt = CBSUtil.decrypt(bodyBytes);
            }else {
                throw new BaseSystemException("业务参考号不能为空！！！！");
            }
        }
        return entity;
    }

    /**
     * 交易明细查询
     *
     * @param entity
     * @return
     */
    @Override
    public SysExtensionApiEntity transactionDetailQuery(SysExtensionApiEntity entity) {
        String bodys = entity.getBodys();
        if (StrUtil.isEmpty(bodys)){
            TransactionDetailReqDTO transactionDetailReqDTO = JSONObject.parseObject(bodys,TransactionDetailReqDTO.class);
            String requestData = JSONObject.toJSONString(transactionDetailReqDTO);
            //签名
            long timestamp = System.currentTimeMillis();
            String sign = CBSUtil.sign(requestData,timestamp);
            //加密
            byte[] encryptedData = CBSUtil.encrypt(requestData);
            Map<String,String> header = getHeaders(sign,timestamp);
            entity.setBodys(Base64.getEncoder().encodeToString(encryptedData));
            entity.setHeaders(header);
        }
        return entity;
    }

    /**
     * 代发代扣
     *
     * @param entity
     * @return
     */
    @Override
    public SysExtensionApiEntity payApplyAgent(SysExtensionApiEntity entity) {
        return null;
    }

    /**
     * 代发代扣结果查询
     *
     * @param entity
     * @return
     */
    @Override
    public SysExtensionApiEntity payApplyAgentResult(SysExtensionApiEntity entity) {
        return null;
    }

    /**
     * 电子回单查询
     *
     * @param entity
     * @return
     */
    @Override
    public SysExtensionApiEntity elecQuery(SysExtensionApiEntity entity) {
        String bodys = entity.getBodys();
        if (StrUtil.isNotEmpty(bodys)){
            ElecRequestDTO elecRequestDTO = JSONObject.parseObject(bodys,ElecRequestDTO.class);
            String requestData = JSONObject.toJSONString(elecRequestDTO);
            //签名
            long timestamp = System.currentTimeMillis();
            String sign = CBSUtil.sign(requestData,timestamp);
            //加密
            byte[] encryptedData = CBSUtil.encrypt(requestData);
            Map<String,String> header = getHeaders(sign,timestamp);
            entity.setBodys(Base64.getEncoder().encodeToString(encryptedData));
            entity.setHeaders(header);
            byte[] bodyBytes = HttpRequest.post("https://cbs8-openapi-reprd.csuat.cmburl.cn/openapi/account/openapi/v1/electronic-bill/query").addHeaders(header).body(encryptedData).execute().bodyBytes();
            //解密报文
            String decrypt = CBSUtil.decrypt(bodyBytes);
            System.out.println(decrypt);
        }
        return entity;
    }



    private Map<String,String> getHeaders(String sign,long timestamp){
        Map<String,String> header = new HashMap();
        header.put(CBSUtil.SIGN_HEADER_NAME,sign);
        header.put(CBSUtil.TIMESTAMP_HEADER,Long.toString(timestamp));
        header.put(HTTP.CONTENT_TYPE,CBSUtil.TARGET_CONTENT_TYPE);
        header.put(CBSUtil.AUTHORIZATION,CBSUtil.BEARER+ CbsAccessToken.getToken());
        return header;
    }
}
