package com.hzya.frame.seeyon.cbs8.service.impl;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.basedao.service.impl.BaseService;
import com.hzya.frame.cbs8.util.PayState;
import com.hzya.frame.seeyon.cbs8.dao.ICbsLogDao;
import com.hzya.frame.seeyon.cbs8.entity.CbsLogEntity;
import com.hzya.frame.seeyon.cbs8.entity.PaymentEntity;
import com.hzya.frame.seeyon.cbs8.service.ICbsLogService;
import com.hzya.frame.web.exception.BaseSystemException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Description cbs8支付日志
 * @Author xiangerlin
 * @Date 2024/6/14 17:22
 **/
@Service()
public class CbsLogServiceImpl extends BaseService<CbsLogEntity,String> implements ICbsLogService {

    Logger log = LoggerFactory.getLogger(getClass());

    private ICbsLogDao cbsLogDao;

    @Autowired
    public void setCbsLogDao(ICbsLogDao cbsLogDao) {
        this.cbsLogDao = cbsLogDao;
        this.dao = cbsLogDao;
    }

    /**
     * 查询支付中的数据
     *
     * @param logEntity
     * @return
     */
    @Override
    public List<CbsLogEntity> queryInPayment(CbsLogEntity logEntity) {
        List<CbsLogEntity> logList = cbsLogDao.queryList(logEntity, "CbsLogEntity_list_base_in_payment");
        return logList;
    }

    /**
     * 保存日志，通过rest接口的形式
     *
     * @param logEntity
     */
    @Override
    public void saveLog(CbsLogEntity logEntity) {

    }

    /**
     * 补推，从自己开发的页面或者APIpost
     * 需要传oa表单id和表单编号
     *
     * @param entity
     */
    @Override
    public void retry(CbsLogEntity entity) {
        PaymentEntity paymentEntity = new PaymentEntity();
        paymentEntity.setOaId(entity.getOa_id());
        paymentEntity.setReferenceNum(entity.getBill_code());
        CbsLogEntity logEntity = new CbsLogEntity();
        logEntity.setId(entity.getId());
        logEntity = cbsLogDao.queryOne(logEntity);
        String pay_state = logEntity.getPay_state();
        if (PayState.h.getValue().equals(pay_state)
                || PayState.three.getValue().equals(pay_state)
                || PayState.k.getValue().equals(pay_state)
                || "推送失败".equals(pay_state)){
            //todo 调用重试方法
        }else {
            throw new BaseSystemException("只允许补推支付失败的记录");
        }
    }

    /**
     * 补推，从OA页面
     * 只需要传日志表id就行
     *
     * @param jsonObject
     */
    @Override
    public void resend(JSONObject jsonObject) {
        if (null != jsonObject && StrUtil.isNotEmpty(jsonObject.getString("id"))){
            String id = jsonObject.getString("id");
            CbsLogEntity cbsLogEntity = new CbsLogEntity();
            cbsLogEntity.setId(id);
            cbsLogEntity =cbsLogDao.queryOne(cbsLogEntity);
            if (null != cbsLogEntity && StrUtil.isNotEmpty(cbsLogEntity.getOa_id()) && StrUtil.isNotEmpty(cbsLogEntity.getBill_code())){
                retry(cbsLogEntity);
            }
        }
    }
}
